@@ -1,3 +1,4 @@
+//go:build !no_zfs
 // +build !no_zfs
 
 /*
@@ -18,4 +19,4 @@
 
 package main
 
-import _ "github.com/containerd/zfs"
+import _ "github.com/containerd/zfs/plugin"
