@@ -20,7 +20,6 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strconv"
@@ -48,9 +47,7 @@ func TestNewBundle(t *testing.T) {
 
 	for i, tc := range tests {
 		t.Run(strconv.Itoa(i), func(t *testing.T) {
-			dir, err := ioutil.TempDir("", "test-new-bundle")
-			require.NoError(t, err, "failed to create test directory")
-			defer os.RemoveAll(dir)
+			dir := t.TempDir()
 			work := filepath.Join(dir, "work")
 			state := filepath.Join(dir, "state")
 			id := fmt.Sprintf("new-bundle-%d", i)
